// -*- C++ -*-
/*!
 * @file  RTC_Camera_Multi.h
 * @brief USBカメラ画像取得,MultiCameraImages出力
 * @date  $Date$
 *
 * @author AIZUK
 * kohno@aizuk.jp
 *
 * $Id$
 */

#ifndef RTC_CAMERA_MULTI_H
#define RTC_CAMERA_MULTI_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>




#include <iostream>
#include <opencv2/opencv.hpp>



/*!
 * @class RTC_Camera_Multi
 * @brief USBカメラ画像取得,MultiCameraImages出力
 *
 * USBカメラ（UVC規格）画像取得
 * カメラ：8台まで対応
 *
 * 出力:MultiCameraImages
 *
 */
class RTC_Camera_Multi
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  RTC_Camera_Multi(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~RTC_Camera_Multi();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * カメラ明るさ調整
   * カメラ８台分　　＊デフォルト値はMCM-4350FISH（マイクロビジョン
   * ）
   * [cam0:cam1:~:cam7]
   * - Name: Camera_BRIGHTNESS Camera_BRIGHTNESS
   * - DefaultValue: [0.5:0.5:0.5:0.5:0.5:0.5:0.5:0.5]
   */
  char m_Camera_BRIGHTNESS[256];
  /*!
   * カメラコントラスト調整
   * カメラ８台分　　＊デフォルト値はMCM-4350FISH（マイクロビジョン
   * ）
   * [cam0:cam1:~:cam7]
   * - Name: Camera_CONTRAST Camera_CONTRAST
   * - DefaultValue: [0.580645:0.580645:0.580645:0.580645:0.580645:0.580645:0.580645:0.580645]
   */
  char m_Camera_CONTRAST[256];
  /*!
   * カメラ彩度調整
   * カメラ８台分　　＊デフォルト値はMCM-4350FISH（マイクロビジョン
   * ）
   * [cam0:cam1:~:cam7]
   * - Name: Camera_SATURATION Camera_SATURATION
   * - DefaultValue: [0.483871:0.483871:0.483871:0.483871:0.483871:0.483871:0.483871:0.483871]
   */
  char m_Camera_SATURATION[256];
  /*!
   * カメラ彩度調整
   * カメラ８台分　　＊デフォルト値はMCM-4350FISH（マイクロビジョン
   * ）
   * [cam0:cam1:~:cam7]
   * - Name: Camera_HUE Camera_HUE
   * - DefaultValue: [0.454545:0.454545:0.454545:0.454545:0.454545:0.454545:0.454545:0.454545]
   */
  char m_Camera_HUE[256];
  /*!
   * カメラガンマ補正 カメラ８台分　*1.0で補正なし
   * [cam0:cam1:~:cam7]　
   * - Name: Camera_GAMMA Camera_GAMMA
   * - DefaultValue: [1.0:1.0:1.0:1.0:1.0:1.0:1.0:1.0]
   */
  char m_Camera_GAMMA[256];
  /*!
   * カメラ画像幅 カメラ８台分
   * [cam0:cam1:~:cam7]　
   * - Name: Camera_WIDTH Camera_WIDTH
   * - DefaultValue: [320:320:320:320:320:320:320:320]
   */
  char m_Camera_WIDTH[256];
  /*!
   * カメラ画像高さ カメラ８台分
   * [cam0:cam1:~:cam7]　
   * - Name: Camera_HEIGHT Camera_HEIGHT
   * - DefaultValue: [240:240:240:240:240:240:240:240]
   */
  char m_Camera_HEIGHT[256];
  /*!
   * カメラFPS カメラ８台分
   * [cam0:cam1:~:cam7]　
   * - Name: Camera_FPS Camera_FPS
   * - DefaultValue: [30:30:30:30:30:30:30:30]
   */
  char m_Camera_FPS[256];
  /*!
   * 歪み補正のON/OFF
   * １：ON　　　　　　　Camera_Dist_Outから補正後のデータ出力
   * ０：OFF　　　　Camera_Dist_Outから出力なし
   * - Name: Camera_Distortion_FLG Camera_Distortion_FLG
   * - DefaultValue: 1
   */
  int m_Camera_Distortion_FLG;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::MultiCameraImages m_Camera_Raw_Out;
  /*!
   */
  OutPort<RTC::MultiCameraImages> m_Camera_Raw_OutOut;
  RTC::MultiCameraImages m_Camera_Dist_Out;
  /*!
   */
  OutPort<RTC::MultiCameraImages> m_Camera_Dist_OutOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>
  float	BRIGHTNESS[9];
  float	CONTRAST[9];
  float	SATURATION[9];
  float	HUE[9];
  double	GAMMA[9];
     int	WIDTH[9];
     int	HEIGHT[9];
     int	FPS[9];
     int	CAM_FLG[9];
     int	CAM_NUM;
     int	len_Raw[9];
     int	len_Dist[9];
  IplImage OUT_img[9];
  // void ;
   virtual RTC::ReturnCode_t Config_Param_Read();
   virtual RTC::ReturnCode_t Config_Param_Set();
   virtual RTC::ReturnCode_t Config_Camera_Read();
   virtual RTC::ReturnCode_t Camera_Cap_Prepara();
 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void RTC_Camera_MultiInit(RTC::Manager* manager);
};

#endif // RTC_CAMERA_MULTI_H
