// -*- C++ -*-
/*!
 * @file  RTC_Camera_Multi.cpp
 * @brief USBカメラ画像取得,MultiCameraImages出力
 * @date $Date$
 *
 * @author AIZUK
 * kohno@aizuk.jp
 *
 * $Id$
 */

#include "RTC_Camera_Multi.h"

cv::FileStorage Fix_File[8];
cv::Mat cameraMatrix[9], distCoeffs[9];
cv::FileNode *param;
cv::VideoCapture cap[9];

  IplImage* Cam_img[9];
  int Width[9];
  int Height[9];
  int len_img[9];
  int img_FLG[9];
  char winname[90];
double t0=0,t1=0,sum_t=0,dt=0;
// Module specification
// <rtc-template block="module_spec">
static const char* rtc_camera_multi_spec[] =
  {
    "implementation_id", "RTC_Camera_Multi",
    "type_name",         "RTC_Camera_Multi",
    "description",       "USBカメラ画像取得,MultiCameraImages出力",
    "version",           "1.0.0",
    "vendor",            "AIZUK",
    "category",          "Camera",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Camera_BRIGHTNESS", "[0.5:0.5:0.5:0.5:0.5:0.5:0.5:0.5]",
    "conf.default.Camera_CONTRAST", "[0.580645:0.580645:0.580645:0.580645:0.580645:0.580645:0.580645:0.580645]",
    "conf.default.Camera_SATURATION", "[0.483871:0.483871:0.483871:0.483871:0.483871:0.483871:0.483871:0.483871]",
    "conf.default.Camera_HUE", "[0.454545:0.454545:0.454545:0.454545:0.454545:0.454545:0.454545:0.454545]",
    "conf.default.Camera_GAMMA", "[1.0:1.0:1.0:1.0:1.0:1.0:1.0:1.0]",
    "conf.default.Camera_WIDTH", "[640:640:640:640:640:640:640:640]",
    "conf.default.Camera_HEIGHT", "[480:480:480:480:480:480:480:480]",
    "conf.default.Camera_FPS", "[30:30:30:30:30:30:30:30]",
    "conf.default.Camera_Distortion_FLG", "1",
    // Widget
    "conf.__widget__.Camera_BRIGHTNESS", "text",
    "conf.__widget__.Camera_CONTRAST", "text",
    "conf.__widget__.Camera_SATURATION", "text",
    "conf.__widget__.Camera_HUE", "text",
    "conf.__widget__.Camera_GAMMA", "text",
    "conf.__widget__.Camera_WIDTH", "text",
    "conf.__widget__.Camera_HEIGHT", "text",
    "conf.__widget__.Camera_FPS", "text",
    "conf.__widget__.Camera_Distortion_FLG", "radio",
    // Constraints
    "conf.__constraints__.Camera_Distortion_FLG", "(1,0)",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RTC_Camera_Multi::RTC_Camera_Multi(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Camera_Raw_OutOut("Camera_Raw_Out", m_Camera_Raw_Out),
    m_Camera_Dist_OutOut("Camera_Dist_Out", m_Camera_Dist_Out)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
RTC_Camera_Multi::~RTC_Camera_Multi()
{
}



RTC::ReturnCode_t RTC_Camera_Multi::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("Camera_Raw_Out", m_Camera_Raw_OutOut);
  addOutPort("Camera_Dist_Out", m_Camera_Dist_OutOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Camera_BRIGHTNESS", m_Camera_BRIGHTNESS, "[0.5:0.5:0.5:0.5:0.5:0.5:0.5:0.5]");
  bindParameter("Camera_CONTRAST", m_Camera_CONTRAST, "[0.580645:0.580645:0.580645:0.580645:0.580645:0.580645:0.580645:0.580645]");
  bindParameter("Camera_SATURATION", m_Camera_SATURATION, "[0.483871:0.483871:0.483871:0.483871:0.483871:0.483871:0.483871:0.483871]");
  bindParameter("Camera_HUE", m_Camera_HUE, "[0.454545:0.454545:0.454545:0.454545:0.454545:0.454545:0.454545:0.454545]");
  bindParameter("Camera_GAMMA", m_Camera_GAMMA, "[1.0:1.0:1.0:1.0:1.0:1.0:1.0:1.0]");
  bindParameter("Camera_WIDTH", m_Camera_WIDTH, "[640:640:640:640:640:640:640:640]");
  bindParameter("Camera_HEIGHT", m_Camera_HEIGHT, "[480:480:480:480:480:480:480:480]");
  bindParameter("Camera_FPS", m_Camera_FPS, "[30:30:30:30:30:30:30:30]");
  bindParameter("Camera_Distortion_FLG", m_Camera_Distortion_FLG, "1");
  // </rtc-template>
  
    m_Camera_Raw_Out.images.length(9);
    m_Camera_Dist_Out.images.length(9);
Camera_Cap_Prepara();
Config_Camera_Read();
//Config_Param_Set();
return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi::onFinalize()
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi::onActivated(RTC::UniqueId ec_id)
{

cv::FileStorage  Fix_File_0("camera47.xml", cv::FileStorage::READ);
Fix_File_0["intrinsic"] >> cameraMatrix[0];
Fix_File_0["distortion"] >> distCoeffs[0];
  Config_Param_Set();


  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi::onDeactivated(RTC::UniqueId ec_id)
{
  cv::destroyAllWindows();
    
  return RTC::RTC_OK;
}

RTC::ReturnCode_t RTC_Camera_Multi::onExecute(RTC::UniqueId ec_id)
{
  int i=0;
  int j=0;
  cv::Mat Cam_Raw[9];
  cv::Mat Cam_Raw_Adj[9];;
  cv::Mat Cam_Dist[9];;
  char winname[90];
  uchar GammaLT[256];
struct timeval ti;
  
  Config_Param_Read();
  
	for(i=0;i<9;i++){
		if(CAM_FLG[i]==1){
			cap[i] >> Cam_Raw[i];
			
			if(GAMMA[i]!=1.0){
			  	for(j=0;j<256;j++){
				     GammaLT[j] = (int)(pow((double)j / 255.0, 1.0 /(double)GAMMA[i]) * 255.0);
			    	}
		    	
	//	      		cvLUT(&src_img,&dst_img,&GammaLT_Mat);
				cv::LUT(
				Cam_Raw[i]
				,cv::Mat(1,256,CV_8U,GammaLT)			
				,Cam_Raw_Adj[i]);
			}else{
				Cam_Raw_Adj[i]=Cam_Raw[i];
			}
			
	      	//	sprintf(winname,"Raw_Adj[%d]",i );
		//	cv::imshow(winname, Cam_Raw_Adj[i]);
	      	
			OUT_img[i]=Cam_Raw_Adj[i];
	      		
	 		//len[i] =OUT_img[i]->nChannels * OUT_img[i]->width * OUT_img[i]->height;
		 	len_Raw[i] =Cam_Raw_Adj[i].channels() * Cam_Raw_Adj[i].cols * Cam_Raw_Adj[i].rows;
		 	
			m_Camera_Raw_Out.images[i].pixels.length(len_Raw[i]);
			m_Camera_Raw_Out.images[i].width  = Cam_Raw_Adj[i].cols;
			m_Camera_Raw_Out.images[i].height = Cam_Raw_Adj[i].rows;
	   		memcpy((void *)&(m_Camera_Raw_Out.images[i].pixels[0]), Cam_Raw_Adj[i].data,len_Raw[i]);
	     
			m_Camera_Raw_OutOut.write();
	
		     	if(m_Camera_Distortion_FLG==1){
		     		cv::undistort(Cam_Raw_Adj[i],Cam_Dist[i],cameraMatrix[0], distCoeffs[0]);
		     		//sprintf(winname,"Dist[%d]",i );
				//cv::imshow(winname, Cam_Dist[i]);
	      		
			 	len_Dist[i] =Cam_Dist[i].channels() * Cam_Dist[i].cols * Cam_Dist[i].rows;
			 	m_Camera_Dist_Out.images[i].pixels.length(len_Dist[i]);
				m_Camera_Dist_Out.images[i].width  = Cam_Dist[i].cols;
				m_Camera_Dist_Out.images[i].height = Cam_Dist[i].rows;
		   		memcpy((void *)&(m_Camera_Dist_Out.images[i].pixels[0]), Cam_Dist[i].data,len_Dist[i]);
		     		
				m_Camera_Dist_OutOut.write();
			}
	
		}
	     
	      	//	sprintf(winname,"Raw[%d:%f]",i,GAMMA[i] );
//			cv::imshow(winname, Cam_Raw_Adj[i]);
			//cvShowImage(winname,&OUT_img[i]);
		
    			
		}
		
	gettimeofday(&ti, NULL);
	t0=(double)(ti.tv_sec)+(double)(ti.tv_usec)*0.001*0.001;

	dt=t0-t1;
	
	//sprintf(time_tex,"FPS%lf",dt);
	printf("⊿ t:%lf\n\r",dt);
	printf("\x1b[0J");

	gettimeofday(&ti, NULL);
	 t1=(double)(ti.tv_sec)+(double)(ti.tv_usec)*0.001*0.001;
//	m_Camera_Raw_OutOut.write();
//	m_Camera_Dist_OutOut.write();
	
	int c = cv::waitKey(2);
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_Camera_Multi::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
//-----------------------------------------------------------------------------------------------
//コンフィギュレーションパラメータ読み取り
//-----------------------------------------------------------------------------------------------
RTC::ReturnCode_t RTC_Camera_Multi::Config_Param_Read()
{
  sscanf(m_Camera_BRIGHTNESS
  	   ,"[%f:%f:%f:%f:%f:%f:%f:%f]"
  	   , &BRIGHTNESS[0] , &BRIGHTNESS[1] , &BRIGHTNESS[2] , &BRIGHTNESS[3]
  	   , &BRIGHTNESS[4] , &BRIGHTNESS[5] , &BRIGHTNESS[6] , &BRIGHTNESS[7] );
  	   
  sscanf(m_Camera_CONTRAST
  	   ,"[%f:%f:%f:%f:%f:%f:%f:%f]"
  	   , &CONTRAST[0] , &CONTRAST[1] , &CONTRAST[2] , &CONTRAST[3]
  	   , &CONTRAST[4] , &CONTRAST[5] , &CONTRAST[6] , &CONTRAST[7] );
  
  sscanf(m_Camera_SATURATION
  	   ,"[%f:%f:%f:%f:%f:%f:%f:%f]"
  	   , &SATURATION[0] , &SATURATION[1] , &SATURATION[2] , &SATURATION[3]
  	   , &SATURATION[4] , &SATURATION[5] , &SATURATION[6] , &SATURATION[7] );
  
  sscanf(m_Camera_HUE
  	   ,"[%f:%f:%f:%f:%f:%f:%f:%f]"
  	   , &HUE[0] , &HUE[1] , &HUE[2] , &HUE[3]
  	   , &HUE[4] , &HUE[5] , &HUE[6] , &HUE[7] );
  	   
  sscanf(m_Camera_GAMMA
  	   ,"[%lf:%lf:%lf:%lf:%lf:%lf:%lf:%lf]"
  	   , &GAMMA[0] , &GAMMA[1] , &GAMMA[2] , &GAMMA[3]
  	   , &GAMMA[4] , &GAMMA[5] , &GAMMA[6] , &GAMMA[7] );
  	   
  sscanf(m_Camera_WIDTH
  	   ,"[%d:%d:%d:%d:%d:%d:%d:%d]"
  	   , &WIDTH[0] , &WIDTH[1] , &WIDTH[2] , &WIDTH[3]
  	   , &WIDTH[4] , &WIDTH[5] , &WIDTH[6] , &WIDTH[7] );
  	   
  sscanf(m_Camera_HEIGHT
  	   ,"[%d:%d:%d:%d:%d:%d:%d:%d]"
  	   , &HEIGHT[0] , &HEIGHT[1] , &HEIGHT[2] , &HEIGHT[3]
  	   , &HEIGHT[4] , &HEIGHT[5] , &HEIGHT[6] , &HEIGHT[7] );
  
  sscanf(m_Camera_FPS
  	   ,"[%d:%d:%d:%d:%d:%d:%d:%d]"
  	   , &FPS[0] , &FPS[1] , &FPS[2] , &FPS[3]
  	   , &FPS[4] , &FPS[5] , &FPS[6] , &FPS[7] );
  
  return RTC::RTC_OK;
}

//-----------------------------------------------------------------------------------------------
//読み取ったコンフィギュレーションパラメータをカメラにセット
//-----------------------------------------------------------------------------------------------
RTC::ReturnCode_t RTC_Camera_Multi::Config_Param_Set()
{

	int i=0;
	for(i=0;i<9;i++){
		if(CAM_FLG[i]==1){
		  	cap[i].set(CV_CAP_PROP_BRIGHTNESS, BRIGHTNESS[i] );//0.5
		    	cap[i].set(CV_CAP_PROP_CONTRAST, CONTRAST[i] );// 0.580645
		    	cap[i].set(CV_CAP_PROP_SATURATION, SATURATION[i] );// 0.483871
		  	cap[i].set(CV_CAP_PROP_HUE, HUE[i] );//0.454545
		    	cap[i].set(CV_CAP_PROP_FRAME_WIDTH, WIDTH[i] );// 640
		    	cap[i].set(CV_CAP_PROP_FRAME_HEIGHT, HEIGHT[i] );// 480
		    	cap[i].set(CV_CAP_PROP_FPS, FPS[i] );// 30
		}
	}
	

  return RTC::RTC_OK;
}
//-----------------------------------------------------------------------------------------------
//カメラパラメータを読み取り
//-----------------------------------------------------------------------------------------------
RTC::ReturnCode_t RTC_Camera_Multi::Config_Camera_Read()
{
	int i=0;
	for(i=0;i<9;i++){
		if(CAM_FLG[i]==1){
		 std::cout << "========================CAM" << i << "===========================" <<std::endl;

		 std::cout << "CV_CAP_PROP_FRAME_WIDTH = " << cap[i].get(CV_CAP_PROP_FRAME_WIDTH) << std::endl;
		 std::cout << "CV_CAP_PROP_FRAME_HEIGHT = " << cap[i].get(CV_CAP_PROP_FRAME_HEIGHT) << std::endl;
		 std::cout << "CV_CAP_PROP_FPS = " << cap[i].get(CV_CAP_PROP_FPS) << std::endl;
		 //std::cout << "CV_CAP_PROP_FORMAT = " << cap[i].get(CV_CAP_PROP_FORMAT) << std::endl;
		 std::cout << "CV_CAP_PROP_BRIGHTNESS = " << cap[i].get(CV_CAP_PROP_BRIGHTNESS ) << std::endl;
		 std::cout << "CV_CAP_PROP_CONTRAST = " << cap[i].get(CV_CAP_PROP_CONTRAST ) << std::endl;
		 std::cout << "CV_CAP_PROP_SATURATION = " << cap[i].get(CV_CAP_PROP_SATURATION ) << std::endl;
		 std::cout << "CV_CAP_PROP_HUE = " << cap[i].get(CV_CAP_PROP_HUE) << std::endl;
		 //std::cout << "CV_CAP_PROP_GAIN = " << cap[i].get(CV_CAP_PROP_GAIN) << std::endl;
		 //std::cout << "CV_CAP_PROP_EXPOSURE = " << cap[i].get(CV_CAP_PROP_EXPOSURE) << std::endl;
		 //std::cout << "CV_CAP_PROP_CONVERT_RGB = " << cap[i].get(CV_CAP_PROP_CONVERT_RGB ) << std::endl;
		 std::cout << "=======================================================" <<std::endl;
		 }
	}
  return RTC::RTC_OK;
}
//-----------------------------------------------------------------------------------------------
//カメラ接続接続数（成功数）確認
//-----------------------------------------------------------------------------------------------
RTC::ReturnCode_t RTC_Camera_Multi::Camera_Cap_Prepara()
{
	int i=0;
	for(i=0;i<9;i++)CAM_FLG[i]=0;
	cap[0] = cv::VideoCapture (0);
	cap[1] = cv::VideoCapture (1);
	cap[2] = cv::VideoCapture (2);
	cap[3] = cv::VideoCapture (3);
	cap[4] = cv::VideoCapture (4);
	cap[5] = cv::VideoCapture (5);
	cap[6] = cv::VideoCapture (6);
	cap[7] = cv::VideoCapture (7);
	
	CAM_NUM=0;
	
	if(cap[0].isOpened()){
  	   if(cap[0].get(CV_CAP_PROP_FRAME_WIDTH) != -1 &&  cap[0].get(CV_CAP_PROP_FRAME_HEIGHT) != -1 ){
		   std::cout<< "Found CAM0" <<std::endl;
		   CAM_FLG[0]=1;
		   CAM_NUM++;
	   }else{ 
	   	  std::cout<< "Can't find CAM0 " <<std::endl;
	   }
	}else{
  	   std::cout<< "Can't find CAM0 " <<std::endl;
	}

	if(cap[1].isOpened()){
  	   if(cap[1].get(CV_CAP_PROP_FRAME_WIDTH) != -1 &&  cap[1].get(CV_CAP_PROP_FRAME_HEIGHT) != -1 ){
		   std::cout<< "Found CAM1" <<std::endl;
		   CAM_FLG[1]=1;
		   CAM_NUM++;
	   }else{ 
	   	  std::cout<< "Can't find CAM1 " <<std::endl;
	   }
	}else{
  	   std::cout<< "Can't find CAM1 " <<std::endl;
	}

	if(cap[2].isOpened()){
  	   if(cap[2].get(CV_CAP_PROP_FRAME_WIDTH) != -1 &&  cap[2].get(CV_CAP_PROP_FRAME_HEIGHT) != -1 ){
		   std::cout<< "Found CAM2" <<std::endl;
		   CAM_FLG[2]=1;
		   CAM_NUM++;
	   }else{ 
	   	  std::cout<< "Can't find CAM2 " <<std::endl;
	   }
	}else{
  	   std::cout<< "Can't find CAM2 " <<std::endl;
	}

	if(cap[3].isOpened()){
  	   if(cap[3].get(CV_CAP_PROP_FRAME_WIDTH) != -1 &&  cap[3].get(CV_CAP_PROP_FRAME_HEIGHT) != -1 ){
		   std::cout<< "Found CAM3" <<std::endl;
		   CAM_FLG[3]=1;
		   CAM_NUM++;
	   }else{ 
	   	  std::cout<< "Can't find CAM3 " <<std::endl;
	   }
	}else{
  	   std::cout<< "Can't find CAM3 " <<std::endl;
	}

	if(cap[4].isOpened()){
  	   if(cap[4].get(CV_CAP_PROP_FRAME_WIDTH) != -1 &&  cap[4].get(CV_CAP_PROP_FRAME_HEIGHT) != -1 ){
		   std::cout<< "Found CAM4" <<std::endl;
		   CAM_FLG[4]=1;
		   CAM_NUM++;
	   }else{ 
	   	  std::cout<< "Can't find CAM4 " <<std::endl;
	   }
	}else{
  	   std::cout<< "Can't find CAM4 " <<std::endl;
	}

	if(cap[5].isOpened()){
  	   if(cap[5].get(CV_CAP_PROP_FRAME_WIDTH) != -1 &&  cap[5].get(CV_CAP_PROP_FRAME_HEIGHT) != -1 ){
		   std::cout<< "Found CAM5" <<std::endl;
		   CAM_FLG[5]=1;
		   CAM_NUM++;
	   }else{ 
	   	  std::cout<< "Can't find CAM5 " <<std::endl;
	   }
	}else{
  	   std::cout<< "Can't find CAM5 " <<std::endl;
	}

	if(cap[6].isOpened()){
	   if(cap[6].get(CV_CAP_PROP_FRAME_WIDTH) != -1 &&  cap[6].get(CV_CAP_PROP_FRAME_HEIGHT) != -1 ){
		   std::cout<< "Found CAM6" <<std::endl;
		   CAM_FLG[6]=1;
		   CAM_NUM++;
	   }else{ 
	   	  std::cout<< "Can't find CAM6 " <<std::endl;
	   }
  	}else{
  	   std::cout<< "Can't find CAM6 " <<std::endl;
	}

	if(cap[7].isOpened()){
  	   if(cap[7].get(CV_CAP_PROP_FRAME_WIDTH) != -1 &&  cap[7].get(CV_CAP_PROP_FRAME_HEIGHT) != -1 ){
		   std::cout<< "Found CAM7" <<std::endl;
		   CAM_FLG[7]=1;
		   CAM_NUM++;
	   }else{ 
	   	  std::cout<< "Can't find CAM7 " <<std::endl;
	   }
	}else{
  	   std::cout<< "Can't find CAM7 " <<std::endl;
	}


         std::cout<< "CAM_NUM : " <<CAM_NUM<< std::endl;
 	return RTC::RTC_OK;
}

extern "C"
{
 
  void RTC_Camera_MultiInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtc_camera_multi_spec);
    manager->registerFactory(profile,
                             RTC::Create<RTC_Camera_Multi>,
                             RTC::Delete<RTC_Camera_Multi>);
  }
  
};


